from pydantic import BaseModel


class ChannelModel(BaseModel):
    ChannelID: int
    TelegramGrpID: int
    OffsetMessageID: int
    ChannelName: str
    SignatureLinesCount: int
    CategoryID: int

    class Config:
        orm_mode = True


class ChannelUpdateModel(BaseModel):
    ChannelID: int
    OffsetMessageID: int
    SignatureLinesCount: int
    CategoryID: str



from sqlalchemy import Column, Integer, DateTime, Text, Boolean, ForeignKey
from sqlalchemy.orm import relationship
from yz_tg_shared.entities.base import Base
from yz_tg_shared.entities.channel import Channel


class Message(Base):
    __tablename__ = 'Messages'
    MessageID = Column(Integer, primary_key=True, autoincrement=True)
    ChannelID = Column(Integer, ForeignKey('Channels.ChannelID'))
    MsgID = Column(Integer, nullable=False)
    MessageDate = Column(DateTime, nullable=False)
    MessageText = Column(Text, nullable=False)
    IsModified = Column(Boolean, default=False)
    channel = relationship('Channel', back_populates='messages')


Channel.messages = relationship('Message', order_by=Message.MessageID, back_populates='channel')



from yz_tg_shared.entities.channel import Channel
from yz_tg_shared.models.channel_model import ChannelUpdateModel


class ChannelRepository:
    def __init__(self, session):
        self.session = session

    def get_all(self):
        return self.session.query(Channel).all()

    def UpdateOffset(self, channel_id: int, channel_offset_message_id: int):
        self.session.query(Channel).filter(Channel.ChannelID == channel_id).update({Channel.OffsetMessageID: channel_offset_message_id},
                                                                                   synchronize_session=False)
        self.session.commit()
        # channel = self.session.query(Channel).filter_by(ChannelID=channel_id).first()
        # if channel:
        #     channel.OffsetMessageID = channel_offset_message_id
        #     self.session.commit()
        #     self.session.refresh(channel)
        # return channel



from pydantic import BaseModel
from typing import Optional


class MessageModel(BaseModel):
    MessageID: int
    MsgID: int
    ChannelID: int
    MessageDate: str
    MessageText: str
    IsModified: Optional[bool]

    class Config:
        orm_mode = True


class MessageCreateModel(BaseModel):
    ChannelID: int
    MsgID: int
    MessageDate: str
    MessageText: str
    IsModified: Optional[bool] = False


class MessageUpdateModel(BaseModel):
    MessageID: int
    MsgID: int
    MessageText: str
    IsModified: Optional[bool] = True



from sqlalchemy import Column, Integer, String, ForeignKey, Boolean
from sqlalchemy.orm import relationship
from yz_tg_shared.entities.base import Base
from yz_tg_shared.entities.category import Category


class Channel(Base):
    __tablename__ = 'Channels'
    ChannelID = Column(Integer, primary_key=True, autoincrement=True)
    TelegramGrpID = Column(Integer, nullable=False)
    OffsetMessageID = Column(Integer, nullable=True)
    ChannelName = Column(String(255), nullable=False)
    CategoryID = Column(Integer, ForeignKey('Categories.CategoryID'))
    SignatureLinesCount = Column(Integer, nullable=True)
    category = relationship('Category', back_populates='channels')


Category.channels = relationship('Channel', order_by=Channel.ChannelID, back_populates='category')



from yz_tg_shared.entities.category import Category


class CategoryRepository:
    def __init__(self, session):
        self.session = session

    def get_all(self):
        return self.session.query(Category).all()



from sqlalchemy import Column, Integer, String, Text
from yz_tg_shared.entities.base import Base


class Category(Base):
    __tablename__ = 'Categories'
    CategoryID = Column(Integer, primary_key=True, autoincrement=True)
    CategoryName = Column(String(255), nullable=False)
    Description = Column(Text)



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from yz_tg_shared.entities.base import Base
from yz_tg_shared.data_access.repositories.category_repository import CategoryRepository
from yz_tg_shared.data_access.repositories.channel_repository import ChannelRepository
from yz_tg_shared.data_access.repositories.message_repository import MessageRepository


class UnitOfWork:
    def __init__(self, db_url):
        self.engine = create_engine(db_url)
        Base.metadata.create_all(self.engine)
        self.Session = sessionmaker(bind=self.engine)

    def __enter__(self):
        self.session = self.Session()
        self.categories = CategoryRepository(self.session)
        self.channels = ChannelRepository(self.session)
        self.messages = MessageRepository(self.session)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type:
            self.session.rollback()
        else:
            self.session.commit()
        self.session.close()



from datetime import datetime

from pydantic.fields import Optional, List

from yz_tg_shared.entities.message import Message
from yz_tg_shared.models.message_model import MessageCreateModel, MessageUpdateModel


class MessageRepository:
    def __init__(self, session):
        self.session = session

    def get_all(self):
        return self.session.query(Message).all()

    def create(self, message_data: MessageCreateModel):
        new_message = Message(
            ChannelID=message_data.ChannelID,
            MessageDate=message_data.MessageDate,
            MsgID=message_data.MsgID,
            MessageText=message_data.MessageText,
            IsModified=message_data.IsModified
        )
        self.session.add(new_message)
        self.session.commit()
        self.session.refresh(new_message)
        return new_message

    def update(self, message_data: MessageUpdateModel):
        message = self.session.query(Message).filter_by(MessageID=message_data.MessageID).first()
        if message:
            message.MessageText = message_data.MessageText
            message.MsgID = message_data.MsgID
            message.IsModified = message_data.IsModified
            self.session.commit()
            self.session.refresh(message)
        return message

    def update_is_hidden(self, message_data: MessageUpdateModel):
        message = self.session.query(Message).filter_by(MessageID=message_data.MessageID).first()
        if message:
            message.IsModified = message_data.IsModified
            self.session.commit()
            self.session.refresh(message)
        return message

    def get_filtered(self, group_id: Optional[int] = None, date_from: Optional[datetime] = None,
                     date_to: Optional[datetime] = None, is_modified: Optional[bool] = None) -> List[Message]:
        query = self.session.query(Message)

        if group_id is not None:
            query = query.filter(Message.GroupID == group_id)

        if date_from is not None:
            query = query.filter(Message.MessageDate >= date_from)

        if date_to is not None:
            query = query.filter(Message.MessageDate <= date_to)

        if is_modified is not None:
            query = query.filter(Message.IsModified == is_modified)

        return query.all()



from setuptools import setup, find_packages

setup(
    name='yz_tg_shared',
    version='1.0.8',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "greenlet==3.0.3",
        "mysql-connector-python==9.0.0",
        "pyaes==1.6.1",
        "pyasn1==0.6.0",
        "pydantic==1.10.17",
        "pydantic-sqlalchemy==0.0.9",
        "SQLAlchemy==1.4.52",
        "typing_extensions==4.12.2"
    ],
    author='OP',
    author_email='parfenchuks@gmail.com',
    description='A package for shared code',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
